/*
 * libxml-dtc provides an XML framework creation facilities
 * Copyright (C) 2003-2004 Mike Hommey <mh@glandium.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include <xml-dtc.h>
#include <hash.h>
#include <string.h>
#include <stdlib.h>

#define SIZE_MOD 64

/* NOT A REAL HASH FOR THE MOMENT (INTERNALLY), BUT ACTS AS IF IT WAS (EXTERNALLY)... */

int xml_dtc_hash_free(xml_dtc_hash *hash) {
  int i;
  if (hash == NULL)
    return -1;

  if (hash->resource)
    xml_dtc_resource_unlink(hash->resource);

  if (hash->list) {
    for (i = 0; i < hash->num; i++) {
      if (hash->unlink_key && hash->list[i].key)
	  hash->unlink_key(hash->list[i].key);
      if (hash->unlink_element && hash->list[i].element)
	  hash->unlink_element(hash->list[i].element);
    }
    free(hash->list);
  }

  free(hash);
  return 0;
}

xml_dtc_hash *xml_dtc_hash_new(const xml_dtc_resource *resource, xml_dtc_hash_unlink_key_callback unlink_key, xml_dtc_hash_link_key_callback link_key, xml_dtc_hash_unlink_element_callback unlink_element, xml_dtc_hash_link_element_callback link_element) {
  xml_dtc_hash *result;

  if (resource == NULL)
    return NULL;
  
  result = (xml_dtc_hash *) malloc(sizeof(xml_dtc_hash));
  memset(result, 0, sizeof(xml_dtc_hash));
  result->resource = xml_dtc_resource_link(resource);

  result->link_key = link_key;
  result->unlink_key = unlink_key;
  result->link_element = link_element;
  result->unlink_element = unlink_element;

  return result;
}

void *xml_dtc_hash_get(xml_dtc_hash *hash, const char *key) {
  if (hash) {
    if (key == NULL)
      return hash->dfault->element;
    if (hash->list) {
      int i;
      for (i = 0; i < hash->num; i++) {
	if (strcmp(key, hash->list[i].key) == 0) {
	  return hash->list[i].element;
	}
      }
    }
  }
  return NULL;
}

char *xml_dtc_hash_get_key(xml_dtc_hash *hash, const char *key) {
  if (hash) {
    if (key == NULL)
      return hash->dfault->key;
    if (hash->list) {
      int i;
      for (i = 0; i < hash->num; i++) {
	if (strcmp(key, hash->list[i].key) == 0) {
	  return hash->list[i].key;
	}
      }
    }
  }
  return NULL;
}

int xml_dtc_hash_add(xml_dtc_hash *hash, char *key, void *element) {
  if (hash && key) {
    int i;
    for (i = 0; i < hash->num; i++) {
      if (strcmp(key, hash->list[i].key) == 0) {
	xml_dtc_msg(hash->resource, XML_DTC_MSG_DEBUG, "Key already in hash: %s.", key);
	return -1;
      }
    }
    if (hash->size <= hash->num) {
      hash->list = realloc(hash->list, sizeof(xml_dtc_hash_struct) * (hash->size + SIZE_MOD));
      /* TODO: Should test realloc result */
      hash->size += SIZE_MOD;
    }
    hash->list[hash->num].key = hash->link_key?hash->link_key(key):key;
    hash->list[hash->num++].element = element?(hash->link_element?hash->link_element(element):element):NULL;

    return 0;
  } else {
    return -1;
  }
}

int xml_dtc_hash_set_default(xml_dtc_hash *hash, const char *key) {
  if (hash) {
    if (hash->list) {
      int i;
      for (i = 0; i < hash->num; i++) {
	if (strcmp(key, hash->list[i].key) == 0) {
	  hash->dfault = &hash->list[i];
	  return 0;
	}
      }
    }
  }
  return -1;
}
