/*
 * libxml-dtc provides an XML framework creation facilities
 * Copyright (C) 2003-2004 Mike Hommey <mh@glandium.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include <xml-dtc.h>
#include <list.h>
#include <string.h>
#include <stdlib.h>

#define SIZE_MOD 64

int xml_dtc_list_free(xml_dtc_list *list) {
  int i;
  if (list == NULL)
    return -1;

  if (list->resource)
    xml_dtc_resource_unlink(list->resource);

  if (list->list) {
    if (list->unlink_element) {
      for (i = 0; i < list->len; i++) {
	if (list->list[i])
	  list->unlink_element(list->list[i]);
      }
    }
    free(list->list);
  }

  free(list);
  return 0;
}

xml_dtc_list *xml_dtc_list_new(const xml_dtc_resource *resource, xml_dtc_list_unlink_element_callback unlink, xml_dtc_list_link_element_callback link) {
  xml_dtc_list *result;

  if (resource == NULL)
    return NULL;
  
  result = (xml_dtc_list *) malloc(sizeof(xml_dtc_list));
  memset(result, 0, sizeof(xml_dtc_list));
  result->resource = xml_dtc_resource_link(resource);
  result->link_element = link;
  result->unlink_element = unlink;

  return result;
}

void *xml_dtc_list_get(xml_dtc_list *list, int index) {
  if (list)
    if (list->list)
      if ((index >= 0) && (index < list->len))
      return list->list[index];
  return NULL;
}

int xml_dtc_list_push(xml_dtc_list *list, void *element) {
  if (list && element) {
    if ((list->len % SIZE_MOD) == 0) {
      list->list = realloc(list->list, sizeof(void *) * (list->len + SIZE_MOD));
      /* TODO: Should test realloc result */
    }
    list->list[list->len++] = element?(list->link_element?list->link_element(element):element):NULL;
    return 0;
  } else {
    return -1;
  }
}

void *xml_dtc_list_pop(xml_dtc_list *list) {
  if (list && list->len) {
    void *result;

    result = list->list[--list->len];
    list->list[list->len] = NULL;

    return result;
  } else {
    return NULL;
  }
}
