/*
 * libxml-dtc provides an XML framework creation facilities
 * Copyright (C) 2003-2004 Mike Hommey <mh@glandium.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#ifndef __RESOURCE_H__
#define __RESOURCE_H__

typedef struct _xml_dtc_resource xml_dtc_resource;

#include <stdarg.h>

#define XML_DTC_MSG_NUM_LEVELS 5
typedef enum _xml_dtc_msg_level xml_dtc_msg_level;
enum _xml_dtc_msg_level {
  XML_DTC_MSG_ERROR = 0,
  XML_DTC_MSG_WARNING = 1,
  XML_DTC_MSG_NOTICE = 2,
  XML_DTC_MSG_INFO = 3,
  XML_DTC_MSG_DEBUG = 4
};

typedef void (* xml_dtc_msg_callback) (void *context, void *channel, const char *format, va_list);

struct _xml_dtc_resource {
  int links;
  xml_dtc_msg_level msg_verbosity;   /* Level of verbosity for messages */
  xml_dtc_msg_callback msg_callback;    /* Messages display callback */
  void *msg_channels[XML_DTC_MSG_NUM_LEVELS];    /* Messages display channels */
  void *msg_context;  /* Messages display context */
};

xml_dtc_resource *xml_dtc_resource_new();

void xml_dtc_msg(const xml_dtc_resource *resource, xml_dtc_msg_level level, const char *format, ...);

xml_dtc_resource *xml_dtc_resource_link(const xml_dtc_resource *resource);

int xml_dtc_resource_unlink(xml_dtc_resource *resource);

#define xml_dtc_resource_set_msg_verbosity(dtc, level) \
   (dtc)->msg_verbosity = (level)

#define xml_dtc_resource_set_msg_callback(dtc, the_callback) \
   (dtc)->msg_callback = (the_callback)

#define xml_dtc_resource_set_msg_channel(dtc, level, channel) \
   (dtc)->msg_channels[level] = (channel);

#define xml_dtc_resource_set_all_msg_channels(dtc, channel) \
   { int i; \
     for(i = 0; i < XML_DTC_MSG_NUM_LEVELS; i++) \
       xml_dtc_resource_set_msg_channel(dtc, i, channel); \
   }

#define xml_dtc_resource_set_msg_context(dtc, ctxt) \
   (dtc)->msg_context = (ctxt)

#endif /* __RESOURCE_H__ */
