/*
 * This module provides redirection to the XML-DTC framework
 * Copyright (C) 2003-2004 Mike Hommey <mh@glandium.org>
 * 
 * This module is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This module is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this module; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#include <modules.h>
#include <xml-dtc.h>
#include <mappings.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>

static int xml_dtc_redirect(xml_dtc_caller *caller, xml_dtc_request *request) {
  char *uri;

  uri = xml_dtc_resolve_mappings(request->mappings, caller->_private);

  request->status = 302;
  request->result = strdup("");
  xml_dtc_request_set_out_header(request, "Location", uri);
  free(uri);

  return 0;
}

static xml_dtc_component *xml_dtc_handler_redirect_callback(xml_dtc_handler *handler, xml_dtc_hash *handlers, xml_dtc_sitemap *sitemap, xmlNodePtr node) {
  xml_dtc_caller *result;
  result = xml_dtc_caller_new(sitemap->resource, "redirector", "redirector");
  result->_private = xmlGetProp(node, "uri");
  result->free_private = (int (*) (void *)) xmlFree;
  result->resolved = 2;
  result->component.handle = xml_dtc_redirect;

  return (xml_dtc_component *)result;
}

static int xml_dtc_redirect_module_register(void) {
  return 0;
}

static const xml_dtc_caller_def xml_dtc_redirector_caller[] = {
  { "redirect-to", "redirector", (xml_dtc_handler_callback) xml_dtc_handler_redirect_callback },
  { NULL }
};

xml_dtc_module xml_dtc_redirect_module = {
  XML_DTC_MODULE_STANDARD_STUFF,
  xml_dtc_redirect_module_register,
  NULL,
  xml_dtc_redirector_caller
};
